==> ubee512-1.4.0.tar.gz: ubee512-1.4.0/src/gui.c <==
//******************************************************************************
//*                                  uBee512                                   *
//*         An emulator for the Microbee Z80 FDD and ROM based models          *
//*                                                                            *
//*                                 GUI module                                 *
//*                                                                            *
//*                       Copyright (C) 2007 Stewart Kay                       *
//******************************************************************************
//
// Provides the graphical interface.
//
//==============================================================================
// ChangeLog (most recent entries are at top)
//==============================================================================
// v1.4.0 - 29 September 2007, S.J.Kay
// - Changes to Y scaling to achieve better aspect ratio.
// - Added changes to error reporting.
//
// v1.0.0 - 21 June 2007, S.J.Kay
// - Added fullscreen toggle mode (ALT+ENTER), special code for win32 is used.
// - Added screen resizing capabilities.
//
// v0.0.0 - 5 June 2007, S.J.Kay
// Start with "nanowasp" source distribution version 0.22. An emulator for the
// microbee 128k. Copyright (C) 2000-2003  David G. Churchill.

//============================================================================== 
/*
 *  uBee512 - An emulator for the Microbee Z80 FDD and ROM based models. 
 *  Copyright (C) 2007  Stewart Kay
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
//==============================================================================   
#include <stdio.h>
#include <stdlib.h>
#include <SDL.h>

#if MINGW
#include <windows.h>
#else
#endif

#include "gui.h"
#include "crtc.h"
#include "ubee512.h"

//==============================================================================
// structures and variables
//==============================================================================
extern int fullscreen;

extern crtc_y_scale;

extern int hdisp;
extern int vdisp;
extern int scans_per_row;

static SDL_Surface *screen;

UINT8 *keystate;

//==============================================================================
// GUI Blit surface.  Display the source surface on the screen surface.
//
//   pass: SDL_Surface src
// return: void
//==============================================================================
void gui_blitsurface (SDL_Surface *src)
{
 SDL_BlitSurface(src, NULL, screen, NULL);
}

//==============================================================================
// GUI Change video.
//
// Will switch to the current mode of window or fullscreen.
//
//   pass: int w		width 
//         int h		height
// return: int			0 if no error, -1 if error
//==============================================================================
int gui_videochange (int w, int h)
{
 h = h * crtc_y_scale;

 if (fullscreen)
    {
     SDL_ShowCursor(SDL_DISABLE);		// don't show the mouse cursor  
     screen = SDL_SetVideoMode(w, h, 16, SDL_FULLSCREEN);
    } 
 else
    {
     SDL_ShowCursor(SDL_ENABLE);		// show the mouse cursor
     screen = SDL_SetVideoMode(w, h, 16, 0);
    } 

 if (screen == NULL)
   {
    console_output();
    printf("gui_videochange: SDL_SetVideoMode failed - %s\n", SDL_GetError());
    return -1;
   }

 return 0;
}

//==============================================================================
// GUI initialise.
//
// This creates a default video mode of 640 x 275 (80x25) which is suitable for
// CP/M operating systems.  The video mode will be able to be changed later when
// the CRTC registers are changed,  typically this will occur when switching
// between CP/M and Microbee's 64x16 BASIC screen.
//
//   pass: void
// return: int			0 if no error, -1 if error
//==============================================================================
int gui_init (void)
{
 return gui_videochange(VIDWIDTH, VIDHEIGHT);
}

//==============================================================================
// GUI de-initialise.
//
//   pass: void
// return: int			0
//==============================================================================
int gui_deinit (void)
{
 return 0;
}

//==============================================================================
// GUI reset.
//
//   pass: void
// return: int			0
//==============================================================================
int gui_reset (void)
{
 return 0;
}

//==============================================================================
// GUI redraw.
//
//   pass: void
// return: void
//==============================================================================
void gui_redraw (void)
{
 crtc_redraw();
 SDL_UpdateRect(screen, 0, 0, 0, 0);	// all 0s updates the entire screen
}

//==============================================================================
// GUI toggle display mode.
// 
// The SDL_WM_ToggleFullScreen(screen) is only for X11 at present according
// to the documentation,  the win32 build uses standard methods to do this.
//
//   pass: void
// return: int			0 if no error, -1 if error
//==============================================================================
int gui_toggledisplay (void)
{
 fullscreen = (! fullscreen);

#ifdef MINGW
 if (gui_videochange(hdisp * 8, vdisp * scans_per_row) == -1)
    return -1;
 crtc_set_redraw();
 gui_redraw(); 
#else
 if (fullscreen)
    SDL_ShowCursor(SDL_DISABLE);	// don't show the mouse cursor  
 else
    SDL_ShowCursor(SDL_ENABLE);		// show the mouse cursor
 SDL_WM_ToggleFullScreen(screen);
#endif

 return 0;
}

//==============================================================================
// Message Box
//
//   pass: int buttons			button combination
//	   char *s			message
// return: int				return value
//==============================================================================
int gui_message_box (int buttons, char *s)
#ifdef MINGW
{
 unsigned int b;
 int res;
 
 switch (buttons)
    {
     case       BUTTON_OK : b = MB_OK;
                            break;
     case BUTTON_OKCANCEL : b = MB_OKCANCEL;     
                            break;
    } 
 res = MessageBox(NULL, s, "uBee512", b | MB_ICONWARNING);
 switch (res)
    {
     case     IDOK : res = BUTTON_IDOK;
                     break;
     case IDCANCEL : res = BUTTON_IDCANCEL;
                     break;
           default : res = 0;
                     break;
    }
 return res;   
}
#else
{
 return BUTTON_IDOK;
} 
#endif

//==============================================================================
// GUI put pixel - (this is inline code)
// 
//   pass: SDL_Surface *surf
//         int x
//         int y
//         Uint32 col
// return: void
//==============================================================================
inline void putpixel (SDL_Surface *surf, int x, int y, Uint32 col)
{
 Uint8 *p = (Uint8 *)surf->pixels + y * surf->pitch + x * 2;
 if ((x < surf->w) && (y < surf->h))
    *(Uint16 *)p = col;
}

==> ubee512-2.0.0.tar.gz: ubee512-2.0.0/src/gui.c <==
//******************************************************************************
//*                                  uBee512                                   *
//*         An emulator for the Microbee Z80 FDD and ROM based models          *
//*                                                                            *
//*                                 GUI module                                 *
//*                                                                            *
//*                       Copyright (C) 2007 Stewart Kay                       *
//******************************************************************************
//
// Provides the graphical interface.
//
//==============================================================================
// ChangeLog (most recent entries are at top)
//==============================================================================
// v2.0.0 - 10 October 2007, S.J.Kay
// - Added message box for Unices builds.
// - Fullscreen switched to a window if message box is displayed then switches
//   back to fullsceen.
//
// v1.4.0 - 29 September 2007, S.J.Kay
// - Changes to Y scaling to achieve better aspect ratio.
// - Added changes to error reporting.
//
// v1.0.0 - 21 June 2007, S.J.Kay
// - Added fullscreen toggle mode (ALT+ENTER), special code for win32 is used.
// - Added screen resizing capabilities.
//
// v0.0.0 - 5 June 2007, S.J.Kay
// Start with "nanowasp" source distribution version 0.22. An emulator for the
// microbee 128k. Copyright (C) 2000-2003  David G. Churchill.

//============================================================================== 
/*
 *  uBee512 - An emulator for the Microbee Z80 FDD and ROM based models. 
 *  Copyright (C) 2007  Stewart Kay
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
//==============================================================================   
#include <stdio.h>
#include <stdlib.h>
#include <SDL.h>

#if MINGW
#include <windows.h>
#else
#include "mbox.h"
#endif

#include "gui.h"
#include "crtc.h"
#include "ubee512.h"

//==============================================================================
// structures and variables
//==============================================================================
extern int fullscreen;

extern int crtc_y_scale;

extern int hdisp;
extern int vdisp;
extern int scans_per_row;

static SDL_Surface *screen;

UINT8 *keystate;

//==============================================================================
// GUI Blit surface.  Display the source surface on the screen surface.
//
//   pass: SDL_Surface src
// return: void
//==============================================================================
void gui_blitsurface (SDL_Surface *src)
{
 SDL_BlitSurface(src, NULL, screen, NULL);
}

//==============================================================================
// GUI Change video.
//
// Will switch to the current mode of window or fullscreen.
//
//   pass: int w		width 
//         int h		height
// return: int			0 if no error, -1 if error
//==============================================================================
int gui_videochange (int w, int h)
{
 h = h * crtc_y_scale;

 if (fullscreen)
    {
     SDL_ShowCursor(SDL_DISABLE);		// don't show the mouse cursor  
     screen = SDL_SetVideoMode(w, h, 16, SDL_FULLSCREEN);
    } 
 else
    {
     SDL_ShowCursor(SDL_ENABLE);		// show the mouse cursor
     screen = SDL_SetVideoMode(w, h, 16, 0);
    } 

 if (screen == NULL)
   {
    console_output();
    printf("gui_videochange: SDL_SetVideoMode failed - %s\n", SDL_GetError());
    return -1;
   }

 return 0;
}

//==============================================================================
// GUI initialise.
//
// This creates a default video mode of 640 x 275 (80x25) which is suitable for
// CP/M operating systems.  The video mode will be able to be changed later when
// the CRTC registers are changed,  typically this will occur when switching
// between CP/M and Microbee's 64x16 BASIC screen.
//
//   pass: void
// return: int			0 if no error, -1 if error
//==============================================================================
int gui_init (void)
{
 return gui_videochange(VIDWIDTH, VIDHEIGHT);
}

//==============================================================================
// GUI de-initialise.
//
//   pass: void
// return: int			0
//==============================================================================
int gui_deinit (void)
{
 return 0;
}

//==============================================================================
// GUI reset.
//
//   pass: void
// return: int			0
//==============================================================================
int gui_reset (void)
{
 return 0;
}

//==============================================================================
// GUI redraw.
//
//   pass: void
// return: void
//==============================================================================
void gui_redraw (void)
{
 crtc_redraw();
 SDL_UpdateRect(screen, 0, 0, 0, 0);	// all 0s updates the entire screen
}

//==============================================================================
// GUI toggle display mode.
// 
// The SDL_WM_ToggleFullScreen(screen) is only for X11 at present according
// to the documentation,  the win32 build uses standard methods to do this.
//
//   pass: void
// return: int			0 if no error, -1 if error
//==============================================================================
int gui_toggledisplay (void)
{
 fullscreen = (! fullscreen);

#ifdef MINGW
 if (gui_videochange(hdisp * 8, vdisp * scans_per_row) == -1)
    return -1;
 crtc_set_redraw();
 gui_redraw(); 
#else
 if (fullscreen)
    SDL_ShowCursor(SDL_DISABLE);	// don't show the mouse cursor  
 else
    SDL_ShowCursor(SDL_ENABLE);		// show the mouse cursor
 SDL_WM_ToggleFullScreen(screen);
#endif

 return 0;
}

//==============================================================================
// Message Box
//
//   pass: int buttons			button combination
//	   char *s			message
// return: int				return value
//==============================================================================
int gui_message_box (int buttons, char *s)
{
 int fs = fullscreen;
 
 if (fs)
    gui_toggledisplay();

#ifdef MINGW
 unsigned int b;
 int res;
 
 switch (buttons)
    {
     case       BUTTON_OK : b = MB_OK;
                            break;
     case BUTTON_OKCANCEL : b = MB_OKCANCEL;     
                            break;
    } 
 res = MessageBox(NULL, s, ICONSTRING, b | MB_ICONWARNING);
 if (fs)
    gui_toggledisplay();

 switch (res)
    {
     case     IDOK : return BUTTON_IDOK;
     case IDCANCEL : return BUTTON_IDCANCEL;
           default : return 0;
    }
#else
 static char *buts[]=
    {	"  OK  ",
        "Cancel",
        NULL
    };
 static char* xargv[]=
    {	ICONSTRING, /* This will come up in the title bar */
    	NULL
    };

 char *result;

 result = mbox(/* number of strings in 'xargv' */ 1, xargv
             ,s, buts
             ,/*make first button '0' the default*/ 0);
 if (fs)
    gui_toggledisplay();

 if (strcmp(result,buts[0]) == 0)
    return BUTTON_IDOK;
 else
    if (strcmp(result,buts[1]) == 0)
       return BUTTON_IDCANCEL;
 /* X Closer returns "delete_window" */
    else
       if (strcmp(result, "delete_window"))
          return BUTTON_IDCANCEL;
       else
          return BUTTON_IDCANCEL;
#endif
}

//==============================================================================
// GUI put pixel - (this is inline code)
// 
//   pass: SDL_Surface *surf
//         int x
//         int y
//         Uint32 col
// return: void
//==============================================================================
inline void putpixel (SDL_Surface *surf, int x, int y, Uint32 col)
{
 Uint8 *p = (Uint8 *)surf->pixels + y * surf->pitch + x * 2;
 if ((x < surf->w) && (y < surf->h))
    *(Uint16 *)p = col;
}
